using HackathonPreparing.ApiService.Features.AuthFeature;
using Microsoft.AspNetCore.Identity;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;
using static HackathonPreparing.ApiService.Features.DevFeature.ReflectionHelpers;

namespace HackathonPreparing.ApiService.Features.DevFeature;

public static class DbHelpersEndpointsExtensions
{
    public static RouteGroupBuilder AddDbHelpersEndpoints(this WebApplication app)
    {
        var group = app.MapGroup("db");

        group.MapPost("/migrate/{contexClassName}", async (string contexClasstName, IServiceProvider services) => 
        {
            var contextType = GetEfCoreClassContexts().FirstOrDefault(c => c.Name == contexClasstName);

            if(contextType is null)
                return Results.NotFound();

            using var scope = services.CreateScope();    

            var dbContext = (DbContext)scope.ServiceProvider.GetRequiredService(contextType);

            await dbContext.Database.MigrateAsync();
            return Results.Ok();
        });
        
        group.MapPost("/migrate", async (IServiceProvider services) => 
        {
            foreach(var contextType in GetEfCoreClassContexts())
            {
                using var scope = services.CreateScope();    

                var dbContext = (DbContext)scope.ServiceProvider.GetRequiredService(contextType);

                await dbContext.Database.MigrateAsync();
            }
            return Results.Ok();
        });


        group.MapPost("/add-moderator", async ([FromServices]UserManager<User> userManager) => {
            var user = new User{
                Email = "m@gmail.com",
                UserName = "m@gmail.com"
            };
            var result = await userManager.CreateAsync(user, "PassPass123_");

            if(!result.Succeeded)
                return Results.BadRequest(result.Errors);
            
            var roleResult = await userManager.AddToRoleAsync(user, "moderator");

            if(!roleResult.Succeeded)
                return Results.BadRequest(roleResult.Errors);

            return Results.Ok(new {
                Login = "m@gmail.com",
                Password = "PassPass123_",
                Role = "moderator"
            });
        }).WithDescription($"Create user with login/email/username: m@gmail.com and password: PassPass123_");

        group.WithTags("db");

        return group;
    }
}